Keen.Dataviz.prototype.parseRequest = function(req){
  this.dataset = _parseRequest.call(this, req);
  // Update the default title every time
  this.view.defaults.title = _getDefaultTitle.call(this, req);
  // Update the active title if not set
  if (!this.title()) this.title(this.view.defaults.title);
  return this;
};

function _parseRequest(req){
  var dataType = _getQueryDataType.call(this, req.queries[0]),
      dataset;

  /*
    TODO: Handle multiple queries
  */
  // First-pass at dataType detection
  this.dataType(dataType);
  if (this.dataType() !== "extraction") {
    // Run data thru raw parser
    dataset = _parseRawData.call(this, (req.data instanceof Array ? req.data[0] : req.data));
  } else {
    // Requires manual parser
    dataset = _parseExtraction.call(this, req);
  }
  return dataset;
}

function _parseExtraction(req){
  var data = (req.data instanceof Array ? req.data[0] : req.data),
      names = req.queries[0].get("property_names") || [],
      schema = { records: "result", select: true };

  if (names) {
    schema.select = [];
    _each(names, function(p){
      schema.select.push({ path: p });
    });
  }

  return new Keen.Dataset(data, schema);
}

// function _parse2xGroupBy(req){}

// function _parseQueryData(query){}

function _getDefaultTitle(req){
  var analysis = req.queries[0].analysis.replace("_", " "),
      collection = req.queries[0].get('event_collection'),
      output;
  output = analysis.replace( /\b./g, function(a){
    return a.toUpperCase();
  });
  if (collection) { output += ' - ' + collection; }
  return output;
}

function _getQueryDataType(query){
  var isInterval = typeof query.params.interval === "string",
      isGroupBy = typeof query.params.group_by === "string",
      is2xGroupBy = query.params.group_by instanceof Array,
      dataType;

  // metric
  if (!isGroupBy && !isInterval) {
    dataType = 'singular';
  }

  // group_by, no interval
  if (isGroupBy && !isInterval) {
    dataType = 'categorical';
  }

  // interval, no group_by
  if (isInterval && !isGroupBy) {
    dataType = 'chronological';
  }

  // interval, group_by
  if (isInterval && isGroupBy) {
    dataType = 'cat-chronological';
  }

  // 2x group_by
  // TODO: research possible dataType options
  if (!isInterval && is2xGroupBy) {
    dataType = 'categorical';
  }

  // interval, 2x group_by
  // TODO: research possible dataType options
  if (isInterval && is2xGroupBy) {
    dataType = 'cat-chronological';
  }

  if (query.analysis === "funnel") {
    dataType = 'cat-ordinal';
  }

  if (query.analysis === "extraction") {
    dataType = 'extraction';
  }
  if (query.analysis === "select_unique") {
    dataType = 'nominal';
  }

  return dataType;
}
